/* This file is automatically generated.  DO NOT EDIT! */
extern ld_emulation_xfer_type ld_elf32mbel_linux_emulation;
extern ld_emulation_xfer_type ld_elf32mb_linux_emulation;

#define EMULATION_LIST \
  &ld_elf32mbel_linux_emulation, \
  &ld_elf32mb_linux_emulation, \
  0
